@@ -1,4 +1,4 @@
-/*find
+/*
     Mac-Telnet - Connect to RouterOS or mactelnetd devices via MAC address
     Copyright (C) 2010, Håkon Nessjøen <haakon.nessjoen@gmail.com>
 
@@ -17,9 +17,12 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 #define _BSD_SOURCE
+#include <libintl.h>
+#include <locale.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <pwd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -41,9 +44,12 @@
 #include "interfaces.h"
 #include "config.h"
 #include "mactelnet.h"
+#include "mndp.h"
 
 #define PROGRAM_NAME "MAC-Telnet"
 
+#define _(String) gettext (String)
+
 static int sockfd = 0;
 static int insockfd;
 
@@ -72,6 +78,7 @@ static int keepalive_counter = 0;
 static unsigned char encryptionkey[128];
 static char username[255];
 static char password[255];
+static char nonpriv_username[255];
 
 struct net_interface interfaces[MAX_INTERFACES];
 struct net_interface *active_interface;
@@ -87,6 +94,30 @@ static void print_version() {
 	fprintf(stderr, PROGRAM_NAME " " PROGRAM_VERSION "\n");
 }
 
+void drop_privileges(char *username) {
+	struct passwd *user = (struct passwd *) getpwnam(username);
+	if (user == NULL) {
+		fprintf(stderr, _("Failed dropping privileges. The user %s is not a valid username on local system.\n"), username);
+		exit(1);
+	}
+	if (getuid() == 0) {
+		/* process is running as root, drop privileges */
+		if (setgid(user->pw_gid) != 0) {
+			fprintf(stderr, _("setgid: Error dropping group privileges\n"));
+			exit(1);
+		}
+		if (setuid(user->pw_uid) != 0) {
+			fprintf(stderr, _("setuid: Error dropping user privileges\n"));
+			exit(1);
+		}
+		/* Verify if the privileges were developed. */
+		if (setuid(0) != -1) {
+			fprintf(stderr, _("Failed to drop privileges\n"));
+			exit(1);
+		}
+	}
+}
+
 static int send_udp(struct mt_packet *packet, int retransmit) {
 	int sent_bytes;
 
@@ -147,7 +178,7 @@ static int send_udp(struct mt_packet *packet, int retransmit) {
 			reset_term();
 		}
 
-		fprintf(stderr, "\nConnection timed out\n");
+		fprintf(stderr, _("\nConnection timed out\n"));
 		exit(1);
 	}
 	return sent_bytes;
@@ -258,7 +289,7 @@ static int handle_packet(unsigned char *data, int data_len) {
 			   the data is raw terminal data to be outputted to the terminal. */
 			else if (cpkt.cptype == MT_CPTYPE_PLAINDATA) {
 				cpkt.data[cpkt.length] = 0;
-				printf("%s", cpkt.data);
+				fputs((const char *)cpkt.data, stdout);
 			}
 
 			/* END_AUTH means that the user/password negotiation is done, and after this point
@@ -296,13 +327,13 @@ static int handle_packet(unsigned char *data, int data_len) {
 		send_udp(&odata, 0);
 
 		if (!quiet_mode) {
-			fprintf(stderr, "Connection closed.\n");
+			fprintf(stderr, _("Connection closed.\n"));
 		}
 
 		/* exit */
 		running = 0;
 	} else {
-		fprintf(stderr, "Unhandeled packet type: %d received from server %s\n", pkthdr.ptype, ether_ntoa((struct ether_addr *)dstmac));
+		fprintf(stderr, _("Unhandeled packet type: %d received from server %s\n"), pkthdr.ptype, ether_ntoa((struct ether_addr *)dstmac));
 		return -1;
 	}
 
@@ -324,7 +355,7 @@ static int find_interface() {
 	bzero(emptymac, ETH_ALEN);
 
 	if (net_get_interfaces(interfaces, MAX_INTERFACES) <= 0) {
-		fprintf(stderr, "Error: No suitable devices found\n");
+		fprintf(stderr, _("Error: No suitable devices found\n"));
 		exit(1);
 	}
 
@@ -396,11 +427,16 @@ int main (int argc, char **argv) {
 	struct sockaddr_in si_me;
 	unsigned char buff[1500];
 	unsigned char print_help = 0, have_username = 0, have_password = 0;
+	unsigned char drop_priv = 0;
 	int c;
 	int optval = 1;
 
+	setlocale(LC_ALL, "");
+	bindtextdomain("mactelnet","/usr/share/locale");
+	textdomain("mactelnet");
+
 	while (1) {
-		c = getopt(argc, argv, "nqt:u:p:vh?");
+		c = getopt(argc, argv, "lnqt:u:p:U:vh?");
 
 		if (c == -1) {
 			break;
@@ -426,6 +462,13 @@ int main (int argc, char **argv) {
 				have_password = 1;
 				break;
 
+			case 'U':
+				/* Save nonpriv_username */
+				strncpy(nonpriv_username, optarg, sizeof(nonpriv_username) - 1);
+				nonpriv_username[sizeof(nonpriv_username) - 1] = '\0';
+				drop_priv = 1;
+				break;
+
 			case 't':
 				connect_timeout = atoi(optarg);
 				break;
@@ -439,6 +482,10 @@ int main (int argc, char **argv) {
 				quiet_mode = 1;
 				break;
 
+			case 'l':
+				return mndp();
+				break;
+
 			case 'h':
 			case '?':
 				print_help = 1;
@@ -448,19 +495,25 @@ int main (int argc, char **argv) {
 	}
 	if (argc - optind < 1 || print_help) {
 		print_version();
-		fprintf(stderr, "Usage: %s <MAC|identity> [-h] [-n] [-t <timeout>] [-u <username>] [-p <password>]\n", argv[0]);
+		fprintf(stderr, _("Usage: %s <MAC|identity> [-h] [-n] [-t <timeout>] [-u <user>] [-p <password>] [-U <user>] | -l\n"), argv[0]);
 
 		if (print_help) {
-			fprintf(stderr, "\nParameters:\n");
-			fprintf(stderr, "  MAC       MAC-Address of the RouterOS/mactelnetd device. Use mndp to discover it.\n");
-			fprintf(stderr, "  identity  The identity/name of your destination device. Uses MNDP protocol to find it.\n");
-			fprintf(stderr, "  -n        Do not use broadcast packets. Less insecure but requires root privileges.\n");
-			fprintf(stderr, "  -t        Amount of seconds to wait for a response on each interface.\n");
-			fprintf(stderr, "  -u        Specify username on command line.\n");
-			fprintf(stderr, "  -p        Specify password on command line.\n");
-			fprintf(stderr, "  -q        Quiet mode.\n");
-			fprintf(stderr, "  -h        This help.\n");
-			fprintf(stderr, "\n");
+			fprintf(stderr, _("\nParameters:\n"
+			"  MAC            MAC-Address of the RouterOS/mactelnetd device. Use mndp to\n"
+			"                 discover it.\n"
+			"  identity       The identity/name of your destination device. Uses\n"
+			"                 MNDP protocol to find it.\n"
+			"  -l             List/Search for routers nearby. (using MNDP)\n"
+			"  -n             Do not use broadcast packets. Less insecure but requires\n"
+			"                 root privileges.\n"
+			"  -t <timeout>   Amount of seconds to wait for a response on each interface.\n"
+			"  -u <user>      Specify username on command line.\n"
+			"  -p <password>  Specify password on command line.\n"
+			"  -U <user>      Drop privileges to this user. Used in conjunction with -n\n"
+			"                 for security.\n"
+			"  -q             Quiet mode.\n"
+			"  -h             This help.\n"
+			"\n"));
 		}
 		return 1;
 	}
@@ -475,11 +528,18 @@ int main (int argc, char **argv) {
 
 	if (use_raw_socket) {
 		if (geteuid() != 0) {
-			fprintf(stderr, "You need to have root privileges to use the -n parameter.\n");
+			fprintf(stderr, _("You need to have root privileges to use the -n parameter.\n"));
 			return 1;
 		}
 
 		sockfd = net_init_raw_socket();
+
+		if (drop_priv) {
+			drop_privileges(nonpriv_username);
+		}
+	} else if (drop_priv) {
+		fprintf(stderr, _("The -U option must be used in conjunction with the -n parameter.\n"));
+		return 1;
 	}
 
 	/* Receive regular udp packets with this socket */
@@ -507,14 +567,14 @@ int main (int argc, char **argv) {
 
 	if (!have_username) {
 		if (!quiet_mode) {
-			printf("Login: ");
+			printf(_("Login: "));
 		}
 		scanf("%254s", username);
 	}
 
 	if (!have_password) {
 		char *tmp;
-		tmp = getpass(quiet_mode ? "" : "Password: ");
+		tmp = getpass(quiet_mode ? "" : _("Password: "));
 		strncpy(password, tmp, sizeof(password) - 1);
 		password[sizeof(password) - 1] = '\0';
 		/* security */
@@ -539,7 +599,7 @@ int main (int argc, char **argv) {
 	setvbuf(stdout, (char*)NULL, _IONBF, 0);
 
 	if (!quiet_mode) {
-		printf("Connecting to %s...", ether_ntoa((struct ether_addr *)dstmac));
+		printf(_("Connecting to %s..."), ether_ntoa((struct ether_addr *)dstmac));
 	}
 
 	/* Initialize receiving socket on the device chosen */
@@ -549,16 +609,16 @@ int main (int argc, char **argv) {
 
 	/* Bind to udp port */
 	if (bind(insockfd, (struct sockaddr *)&si_me, sizeof(si_me)) == -1) {
-		fprintf(stderr, "Error binding to %s:%d, %s\n", inet_ntoa(si_me.sin_addr), sourceport, strerror(errno));
+		fprintf(stderr, _("Error binding to %s:%d, %s\n"), inet_ntoa(si_me.sin_addr), sourceport, strerror(errno));
 		return 1;
 	}
 
 	if (!find_interface() || (result = recvfrom(insockfd, buff, 1400, 0, 0, 0)) < 1) {
-		fprintf(stderr, "Connection failed.\n");
+		fprintf(stderr, _("Connection failed.\n"));
 		return 1;
 	}
 	if (!quiet_mode) {
-		printf("done\n");
+		printf(_("done\n"));
 	}
 
 	/* Handle first received packet */
